#ifndef LED_cpp
#define LED_cpp

#include <Arduino.h>
#include "LEDOutput.cpp"

class LED{
	LEDOutput* _output;
	bool _on;
	unsigned long _fadeDurationOn, _fadeDurationOff, _fadeStart, _fadeEnd;

	unsigned long getFadeDuration(){
		return _on ? _fadeDurationOn : _fadeDurationOff;
	}

	unsigned long getRemainingFadeTime(){
		unsigned long now = millis();
		return _fadeEnd >= now ? constrain(_fadeEnd - now, 0, getFadeDuration()) : 0;
	}

	float getMultiplier() {
		float value = getRemainingFadeTime() / (float)getFadeDuration();
		return 1.0f - value;
	}

	float getOutputMultiplier(){
		float value = getMultiplier();
		return _on ? value : 1.0f - value;
	}

	void reset(bool on){
		if (on == _on)
			return;
		float oldMultiplier = getMultiplier();
		_on = on;
		_fadeStart = millis();
		_fadeEnd = _fadeStart + (getFadeDuration() * oldMultiplier);
	}

public:
	LED(LEDOutput* output, unsigned long fadeDurationOn, unsigned long fadeDurationOff) {
		_output = output;
		_fadeDurationOn = fadeDurationOn;
		_fadeDurationOff = fadeDurationOff;
	}

	void on(){
		reset(true);
	}

	void off(){
		reset(false);
	}

	void toggle(){
		reset(!_on);
	}

	void loop() {
		_output->writeFraction(getOutputMultiplier());
	}
};

#endif
